/*
Copyright 2009 Matthew Woehlke <mw.triad@users.sourceforge.net>
Copyright 2009 Long Huynh Huu <long.upcase@googlemail.com>
Copyright 2003 Sandro Giessl <ceebx@users.sourceforge.net>

originally based on the Keramik configuration dialog:
Copyright 2003 Maksim Orlovich <maksim.orlovich@kdemail.net>

Permission is hereby granted, free of charge, to any person obtaining a
copy of this software and associated documentation files (the "Software"),
to deal in the Software without restriction, including without limitation
the rights to use, copy, modify, merge, publish, distribute, sublicense,
and/or sell copies of the Software, and to permit persons to whom the
Software is furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL
THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
DEALINGS IN THE SOFTWARE.

*/

#include "nitrogenstyleconfig.h"
#include "nitrogenstyleconfig.moc"
#include "nitrogenstyleconfigdata.h"

#include <QtGui/QLayout>
#include <QtGui/QLabel>

#include <KGlobal>
#include <KLocale>
#include <KSharedConfig>
#include <KTabWidget>
#include <KConfigGroup>
#include <kdemacros.h>

#define SCROLLBAR_DEFAULT_WIDTH 15
#define SCROLLBAR_MINIMUM_WIDTH 10
#define SCROLLBAR_MAXIMUM_WIDTH 30

extern "C"
{
  KDE_EXPORT QWidget* allocate_kstyle_config(QWidget* parent)
  {
    KGlobal::locale()->insertCatalog("kstyle_config");
    return new NitrogenStyleConfig(parent);
  }
}

//______________________________________________________________
NitrogenStyleConfig::NitrogenStyleConfig(QWidget* parent): QWidget(parent)
{
  KGlobal::locale()->insertCatalog("kstyle_config");

  /* Stop 1+2: Set up the UI */
  setupUi();

  /* Stop 3: Set up the configuration struct and your widget */
  _toolBarDrawItemSeparator->setChecked( NitrogenStyleConfigData::toolBarDrawItemSeparator() );
  _checkDrawX->setChecked( NitrogenStyleConfigData::checkBoxStyle() == NitrogenStyleConfigData::CS_X );
  _viewDrawTriangularExpander->setChecked( NitrogenStyleConfigData::viewDrawTriangularExpander() );
  _viewDrawTreeBranchLines->setChecked(NitrogenStyleConfigData::viewDrawTreeBranchLines() );
  _scrollBarWidth->setValue( qMin(SCROLLBAR_MAXIMUM_WIDTH, qMax(SCROLLBAR_MINIMUM_WIDTH, NitrogenStyleConfigData::scrollBarWidth())) );
  _scrollBarColored->setChecked( NitrogenStyleConfigData::scrollBarColored() );
  _menuHighlightDark->setChecked( NitrogenStyleConfigData::menuHighlightMode() == NitrogenStyleConfigData::MM_DARK );
  _menuHighlightStrong->setChecked( NitrogenStyleConfigData::menuHighlightMode() == NitrogenStyleConfigData::MM_STRONG || NitrogenStyleConfigData::menuHighlightMode() == NitrogenStyleConfigData::MM_SUBTLE );
  _menuHighlightSubtle->setChecked( NitrogenStyleConfigData::menuHighlightMode() == NitrogenStyleConfigData::MM_SUBTLE );
  _menuHighlightSubtle->setEnabled( NitrogenStyleConfigData::menuHighlightMode() == NitrogenStyleConfigData::MM_STRONG || NitrogenStyleConfigData::menuHighlightMode() == NitrogenStyleConfigData::MM_SUBTLE );

  switch(  NitrogenStyleConfigData::tabStyle() )
  {

    case NitrogenStyleConfigData::TS_PLAIN:
    _tabStyle->setCurrentIndex( 1 );
    break;

    case NitrogenStyleConfigData::TS_SINGLE:
    default:
    _tabStyle->setCurrentIndex( 0 );
    break;

  }

  _animations->setChecked( NitrogenStyleConfigData::animations() );
  _animationsDuration->setValue( NitrogenStyleConfigData::animationsDuration() );
  _animationsDuration->setValue( NitrogenStyleConfigData::animationsDuration() );
  _stackedWidgetAnimations->setChecked( NitrogenStyleConfigData::stackedWidgetAnimations() );

  switch( NitrogenStyleConfigData::menuBarAnimation() )
  {

    case NitrogenStyleConfigData::MB_FOLLOW_MOUSE:
    _menuBarAnimation->setCurrentIndex( 1 );
    break;

    case NitrogenStyleConfigData::MB_NONE:
    _menuBarAnimation->setCurrentIndex( 2 );
    break;

    case NitrogenStyleConfigData::MB_FADE:
    default:
    _menuBarAnimation->setCurrentIndex( 0 );
    break;

  }

  switch( NitrogenStyleConfigData::menuAnimation() )
  {

    case NitrogenStyleConfigData::MA_FOLLOW_MOUSE:
    _menuAnimation->setCurrentIndex( 1 );
    break;

    case NitrogenStyleConfigData::MA_NONE:
    _menuAnimation->setCurrentIndex( 2 );
    break;

    case NitrogenStyleConfigData::MA_FADE:
    default:
    _menuAnimation->setCurrentIndex( 0 );
    break;

  }

  switch( NitrogenStyleConfigData::toolBarAnimation() )
  {

    case NitrogenStyleConfigData::TB_FOLLOW_MOUSE:
    _toolBarAnimation->setCurrentIndex( 1 );
    break;

    case NitrogenStyleConfigData::TB_FADE:
    default:
    _toolBarAnimation->setCurrentIndex( 0 );
    break;

  }

  /* Stop 4: Emit a signal on changes */
  connect( _toolBarDrawItemSeparator, SIGNAL( toggled(bool) ), SLOT( updateChanged() ) );
  connect( _checkDrawX, SIGNAL( toggled(bool) ), SLOT( updateChanged() ) );
  connect( _viewDrawTriangularExpander, SIGNAL( toggled(bool) ), SLOT( updateChanged() ) );
  connect( _viewDrawTreeBranchLines, SIGNAL( toggled(bool) ), SLOT( updateChanged() ) );
  connect( _scrollBarColored, SIGNAL( toggled(bool) ), SLOT( updateChanged() ) );
  connect( _scrollBarWidth, SIGNAL( valueChanged(int) ), SLOT( updateChanged() ) );
  connect( _menuHighlightDark, SIGNAL( toggled(bool) ), SLOT( updateChanged() ) );
  connect( _menuHighlightStrong, SIGNAL( toggled(bool) ), SLOT( updateChanged() ) );
  connect( _menuHighlightSubtle, SIGNAL( toggled(bool) ), SLOT( updateChanged() ) );
  connect( _tabStyle, SIGNAL( currentIndexChanged( int )), SLOT( updateChanged() ) );

  connect( _animations, SIGNAL( toggled(bool) ), SLOT( updateChanged() ) );
  connect( _animationsSlider, SIGNAL( valueChanged(int) ), SLOT( updateChanged() ) );
  connect( _animationsDuration, SIGNAL( valueChanged(int) ), SLOT( updateChanged() ) );
  connect( _menuBarAnimation, SIGNAL( currentIndexChanged( int )), SLOT( updateChanged() ) );
  connect( _stackedWidgetAnimations, SIGNAL( toggled( bool )), SLOT( updateChanged() ) );
  connect( _menuAnimation, SIGNAL( currentIndexChanged( int )), SLOT( updateChanged() ) );
  connect( _toolBarAnimation, SIGNAL( currentIndexChanged( int )), SLOT( updateChanged() ) );

}

//______________________________________________________________
NitrogenStyleConfig::~NitrogenStyleConfig()
{}

//______________________________________________________________
void NitrogenStyleConfig::setupUi( void )
{

  setLayout( new QVBoxLayout() );
  layout()->setMargin(0);

  // tab widget for basic and advanced mode
  KTabWidget* tab( new KTabWidget( this ) );
  layout()->addWidget( tab );

  // general
  {
    QWidget *widget = new QWidget();
    int index = tab->addTab( widget, i18n("&General") );
    tab->setTabToolTip( index, i18n( "Basic widget style configuration options" ) );

    QVBoxLayout* vboxLayout = new QVBoxLayout();
    widget->setLayout( vboxLayout );

    vboxLayout->addWidget( _toolBarDrawItemSeparator = new QCheckBox(widget) );
    _toolBarDrawItemSeparator->setObjectName(QString::fromUtf8("_toolBarDrawItemSeparator"));
    _toolBarDrawItemSeparator->setText(i18n("Draw toolbar item separators"));

    vboxLayout->addWidget( _checkDrawX = new QCheckBox(widget) );
    _checkDrawX->setObjectName(QString::fromUtf8("_checkDrawX"));
    _checkDrawX->setText(i18n("Use 'X' in checkboxes instead of check"));

    vboxLayout->addStretch(1);

  }

  // views
  {
    QWidget *widget = new QWidget();
    int index = tab->addTab( widget, i18n("&Lists") );
    tab->setTabToolTip( index, i18n( "Configuration options for item lists organized in a tree" ) );

    QVBoxLayout* vboxLayout = new QVBoxLayout();
    widget->setLayout( vboxLayout );

    vboxLayout->addWidget( _viewDrawTriangularExpander = new QCheckBox(widget) );
    _viewDrawTriangularExpander->setObjectName(QString::fromUtf8("_viewDrawTriangularExpander"));
    _viewDrawTriangularExpander->setText(i18n("Use triangle tree expander instead of +/-"));

    vboxLayout->addWidget( _viewDrawTreeBranchLines = new QCheckBox(widget) );
    _viewDrawTreeBranchLines->setObjectName(QString::fromUtf8("_viewDrawTreeBranchLines"));
    _viewDrawTreeBranchLines->setText(i18n("Draw tree branch lines"));

    vboxLayout->addStretch(1);
  }

  // tabs
  {
    QWidget *widget = new QWidget();
    tab->addTab( widget, i18n("&Tabs") );

    QGridLayout* gridLayout( new QGridLayout() );
    widget->setLayout( gridLayout );

    QLabel* label(0);
    gridLayout->addWidget( label = new QLabel( i18n( "Tab style:" ), widget ), 0, 0, 1, 1, Qt::AlignRight|Qt::AlignVCenter );
    gridLayout->addWidget( _tabStyle = new KComboBox(widget), 0,1, 1, 1 );
    _tabStyle->setObjectName(QString::fromUtf8("_tabStyle"));
    _tabStyle->clear();
    _tabStyle->insertItems(0, QStringList()
      << i18n("Single")
      << i18n("Plain")
      );
    label->setBuddy( _tabStyle );

    gridLayout->setColumnStretch( 0, 0 );
    gridLayout->setColumnStretch( 1, 0 );
    gridLayout->setColumnStretch( 2, 1 );
    gridLayout->setRowStretch( 0, 0 );
    gridLayout->setRowStretch( 1, 1 );

  }

  // scrollbars
  {
    QWidget *widget = new QWidget();
    tab->addTab( widget, i18n("&Scrollbars") );

    QGridLayout* gridLayout( new QGridLayout() );
    widget->setLayout( gridLayout );

    QLabel* label(0);
    gridLayout->addWidget( _scrollBarColored = new QCheckBox(widget), 0, 0, 1, 3 );
    _scrollBarColored->setObjectName(QString::fromUtf8("_scrollBarColored"));
    _scrollBarColored->setText( i18n( "Colorful hovered scrollbars" ) );
    gridLayout->addWidget( label = new QLabel( i18n( "Width:" ), widget ), 1, 0, 1, 1 );
    gridLayout->addWidget( _scrollBarWidthSlider = new QSlider(widget), 1, 1, 1, 1 );
    _scrollBarWidthSlider->setObjectName(QString::fromUtf8("_scrollBarWidthSlider"));
    _scrollBarWidthSlider->setMinimum(10);
    _scrollBarWidthSlider->setMaximum(30);
    _scrollBarWidthSlider->setPageStep(5);
    _scrollBarWidthSlider->setOrientation(Qt::Horizontal);
    _scrollBarWidthSlider->setTickPosition(QSlider::TicksBelow);
    _scrollBarWidthSlider->setTickInterval(5);
    label->setBuddy( _scrollBarWidthSlider );

    gridLayout->addWidget( _scrollBarWidth = new QSpinBox(widget), 1, 2, 1, 1 );
    _scrollBarWidth->setObjectName(QString::fromUtf8("_scrollBarWidth"));
    _scrollBarWidth->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);
    _scrollBarWidth->setMinimum(10);
    _scrollBarWidth->setMaximum(30);

    connect(_scrollBarWidthSlider, SIGNAL(valueChanged(int)), _scrollBarWidth, SLOT(setValue(int)));
    connect(_scrollBarWidth, SIGNAL(valueChanged(int)), _scrollBarWidthSlider, SLOT(setValue(int)));

    gridLayout->setColumnStretch( 0, 0 );
    gridLayout->setColumnStretch( 1, 0 );
    gridLayout->setColumnStretch( 2, 0 );
    gridLayout->setColumnStretch( 3, 1 );
    gridLayout->setRowStretch( 0, 0 );
    gridLayout->setRowStretch( 1, 0 );
    gridLayout->setRowStretch( 2, 1 );

  }

  // menu
  {
    QWidget *widget = new QWidget();
    tab->addTab( widget, i18n("&Menu Highlight") );

    QGridLayout* gridLayout( new QGridLayout() );
    widget->setLayout( gridLayout );

    gridLayout->addWidget( _menuHighlightDark = new QRadioButton(widget), 0, 0, 1, 2 );
    _menuHighlightDark->setObjectName(QString::fromUtf8("_menuHighlightDark"));
    _menuHighlightDark->setText(i18n("Dark"));

    gridLayout->addWidget( _menuHighlightStrong = new QRadioButton(widget), 1, 0, 1, 2 );
    _menuHighlightStrong->setObjectName(QString::fromUtf8("_menuHighlightStrong"));
    _menuHighlightStrong->setText(i18n("Use selection color"));

    gridLayout->addItem( new QSpacerItem(20, 20, QSizePolicy::Fixed, QSizePolicy::Minimum), 2, 0, 1, 1 );
    gridLayout->addWidget( _menuHighlightSubtle = new QCheckBox(widget), 2, 1, 1, 1 );
    _menuHighlightSubtle->setObjectName(QString::fromUtf8("_menuHighlightSubtle"));
    _menuHighlightSubtle->setText(i18n("Subtle"));

    connect(_menuHighlightStrong, SIGNAL(toggled(bool)), _menuHighlightSubtle, SLOT(setEnabled(bool)));
    connect(_menuHighlightDark, SIGNAL(toggled(bool)), _menuHighlightSubtle, SLOT(setDisabled(bool)));

    gridLayout->setColumnStretch( 0, 0 );
    gridLayout->setColumnStretch( 1, 0 );
    gridLayout->setColumnStretch( 2, 1 );
    gridLayout->setRowStretch( 0, 0 );
    gridLayout->setRowStretch( 1, 0 );
    gridLayout->setRowStretch( 2, 0 );
    gridLayout->setRowStretch( 3, 1 );

  }

  // animations
  {
    QWidget *widget = new QWidget();
    tab->addTab( widget, i18n("&Animations") );


    QGridLayout* gridLayout( new QGridLayout() );
    widget->setLayout( gridLayout );

    gridLayout->addWidget( _animations = new QCheckBox(widget), 0, 0, 1, 3 );
    _animations->setObjectName(QString::fromUtf8("_animations"));
    _animations->setText( i18n( "enable/disable widget animations" ) );
    _animations->setChecked( true );

    QLabel* label(0);
    gridLayout->addWidget( label = new QLabel( i18n( "Duration:" ), widget ), 1, 0, 1, 1 );
    gridLayout->addWidget( _animationsSlider = new QSlider(widget), 1, 1, 1, 1 );
    _animationsSlider->setObjectName(QString::fromUtf8("_animationsSlider"));
    _animationsSlider->setMinimum(50);
    _animationsSlider->setMaximum(500);
    _animationsSlider->setPageStep(100);
    _animationsSlider->setOrientation(Qt::Horizontal);
    label->setBuddy( _animationsSlider );

    gridLayout->addWidget( _animationsDuration = new QSpinBox(widget), 1, 2, 1, 1 );
    _animationsDuration->setObjectName(QString::fromUtf8("_animationsDuration"));
    _animationsDuration->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);
    _animationsDuration->setMinimum(50);
    _animationsDuration->setMaximum(1000);


    gridLayout->addWidget( _stackedWidgetAnimations = new QCheckBox(widget), 2, 0, 1, 3 );
    _stackedWidgetAnimations->setObjectName(QString::fromUtf8("_stackedWidgetAnimations"));
    _stackedWidgetAnimations->setText( i18n( "enable/disable smooth transition between stacked widgets" ) );
    _stackedWidgetAnimations->setChecked( true );
    _stackedWidgetAnimations->hide();

    gridLayout->addWidget( label = new QLabel( i18n( "Menu bar animation type:" ), widget ), 3, 0, 1, 1 );
    gridLayout->addWidget( _menuBarAnimation = new KComboBox(widget), 3, 1, 1, 1 );
    _menuBarAnimation->setObjectName(QString::fromUtf8("_menuBarAnimation"));
    _menuBarAnimation->clear();
    _menuBarAnimation->insertItems(0, QStringList()
      << i18n("Fade")
      << i18n("Follow Mouse")
      << i18n("None")
      );
    label->setBuddy( _menuBarAnimation );

    gridLayout->addWidget(  label = new QLabel( i18n( "Menu animation type:" ), widget ), 4, 0, 1, 1 );
    gridLayout->addWidget( _menuAnimation = new KComboBox(widget), 4, 1, 1, 1 );
    _menuAnimation->setObjectName(QString::fromUtf8("_menuAnimation"));
    _menuAnimation->clear();
    _menuAnimation->insertItems(0, QStringList()
      << i18n("Fade")
      << i18n("Follow Mouse")
      << i18n("None")
      );
    label->setBuddy( _menuAnimation );

    gridLayout->addWidget(  label = new QLabel( i18n( "Toolbar animation type:" ), widget ), 5, 0, 1, 1 );
    gridLayout->addWidget( _toolBarAnimation = new KComboBox(widget), 5, 1, 1, 1 );
    _toolBarAnimation->setObjectName(QString::fromUtf8("_toolBarAnimation"));
    _toolBarAnimation->clear();
    _toolBarAnimation->insertItems(0, QStringList()
      << i18n("Fade")
      << i18n("Follow Mouse")
      );

    label->setBuddy( _toolBarAnimation );
    connect(_animationsSlider, SIGNAL(valueChanged(int)), _animationsDuration, SLOT(setValue(int)));
    connect(_animationsDuration, SIGNAL(valueChanged(int)), _animationsSlider, SLOT(setValue(int)));
    connect(_animations, SIGNAL(toggled(bool)), _animationsSlider, SLOT(setEnabled(bool)));
    connect(_animations, SIGNAL(toggled(bool)), _animationsDuration, SLOT(setEnabled(bool)));
    connect(_animations, SIGNAL(toggled(bool)), _stackedWidgetAnimations, SLOT(setEnabled(bool)));
    connect(_animations, SIGNAL(toggled(bool)), _menuBarAnimation, SLOT(setEnabled(bool)));
    connect(_animations, SIGNAL(toggled(bool)), _menuAnimation, SLOT(setEnabled(bool)));
    connect(_animations, SIGNAL(toggled(bool)), _toolBarAnimation, SLOT(setEnabled(bool)));

    gridLayout->setColumnStretch( 0, 0 );
    gridLayout->setColumnStretch( 1, 0 );
    gridLayout->setColumnStretch( 2, 0 );
    gridLayout->setColumnStretch( 3, 1 );
    gridLayout->setRowStretch( 0, 0 );
    gridLayout->setRowStretch( 1, 0 );
    gridLayout->setRowStretch( 2, 0 );
    gridLayout->setRowStretch( 3, 0 );
    gridLayout->setRowStretch( 4, 0 );
    gridLayout->setRowStretch( 5, 0 );
    gridLayout->setRowStretch( 6, 1 );


  }

  setMinimumSize( QSize( 450, 150 ) );

}

//______________________________________________________________
void NitrogenStyleConfig::save()
{
  /* Stop 5: Save the configuration */
  NitrogenStyleConfigData::setToolBarDrawItemSeparator( _toolBarDrawItemSeparator->isChecked() );
  NitrogenStyleConfigData::setCheckBoxStyle( ( _checkDrawX->isChecked() ? NitrogenStyleConfigData::CS_X : NitrogenStyleConfigData::CS_CHECK ) );
  NitrogenStyleConfigData::setViewDrawTriangularExpander( _viewDrawTriangularExpander->isChecked() );
  NitrogenStyleConfigData::setViewDrawTreeBranchLines( _viewDrawTreeBranchLines->isChecked() );
  NitrogenStyleConfigData::setScrollBarColored( _scrollBarColored->isChecked() );
  NitrogenStyleConfigData::setScrollBarWidth( _scrollBarWidth->value() );
  NitrogenStyleConfigData::setMenuHighlightMode( menuMode() );
  NitrogenStyleConfigData::setTabStyle( tabStyle() );
  NitrogenStyleConfigData::setAnimations( _animations->isChecked() );
  NitrogenStyleConfigData::setAnimationsDuration( _animationsDuration->value() );
  NitrogenStyleConfigData::setStackedWidgetAnimations( _stackedWidgetAnimations->isChecked() );
  NitrogenStyleConfigData::setMenuBarAnimation( menuBarAnimation() );
  NitrogenStyleConfigData::setMenuAnimation( menuAnimation() );
  NitrogenStyleConfigData::setToolBarAnimation( toolBarAnimation() );

  NitrogenStyleConfigData::self()->writeConfig();
}

//______________________________________________________________
void NitrogenStyleConfig::defaults()
{

  /* Stop 6: Set defaults */
  _toolBarDrawItemSeparator->setChecked(true);
  _checkDrawX->setChecked(false);
  _viewDrawTriangularExpander->setChecked(false);
  _viewDrawTreeBranchLines->setChecked(true);
  _scrollBarColored->setChecked(false);
  _scrollBarWidth->setValue(SCROLLBAR_DEFAULT_WIDTH);
  _menuHighlightSubtle->setChecked(false);
  _menuHighlightDark->setChecked(true);
  _tabStyle->setCurrentIndex(0);
  _animations->setChecked( true );
  _animationsDuration->setValue( 200 );
  _stackedWidgetAnimations->setChecked( false );
  _menuBarAnimation->setCurrentIndex(0);
  _menuAnimation->setCurrentIndex(0);
  _toolBarAnimation->setCurrentIndex(0);

  //updateChanged would be done by setChecked already
}

//______________________________________________________
void NitrogenStyleConfig::updateChanged()
{
  /* Stop 7: Check if some value changed */
  if (
    (_toolBarDrawItemSeparator->isChecked() == NitrogenStyleConfigData::toolBarDrawItemSeparator())
    && (_viewDrawTriangularExpander->isChecked() == NitrogenStyleConfigData::viewDrawTriangularExpander())
    && (_viewDrawTreeBranchLines->isChecked() == NitrogenStyleConfigData::viewDrawTreeBranchLines())
    && (_scrollBarColored->isChecked() == NitrogenStyleConfigData::scrollBarColored())
    && (_scrollBarWidth->value() == NitrogenStyleConfigData::scrollBarWidth())
    && ((_checkDrawX->isChecked() ? NitrogenStyleConfigData::CS_X : NitrogenStyleConfigData::CS_CHECK) == NitrogenStyleConfigData::checkBoxStyle())
    && (menuMode() == NitrogenStyleConfigData::menuHighlightMode())
    && (tabStyle() == NitrogenStyleConfigData::tabStyle()
    && (_animations->isChecked() == NitrogenStyleConfigData::animations() )
    && (_animationsDuration->value() == NitrogenStyleConfigData::animationsDuration() )
    && (_stackedWidgetAnimations->isChecked() == NitrogenStyleConfigData::stackedWidgetAnimations() )
    && (menuBarAnimation() == NitrogenStyleConfigData::menuBarAnimation() )
    && (menuAnimation() == NitrogenStyleConfigData::menuAnimation() )
    && (toolBarAnimation() == NitrogenStyleConfigData::toolBarAnimation() )
   )
    )
  {
    emit changed(false);

  } else emit changed(true);

}

//______________________________________________________
int NitrogenStyleConfig::menuMode() const
{
  if (_menuHighlightDark->isChecked()) return NitrogenStyleConfigData::MM_DARK;
  else if (_menuHighlightSubtle->isChecked()) return NitrogenStyleConfigData::MM_SUBTLE;
  else return NitrogenStyleConfigData::MM_STRONG;
}

//______________________________________________________
int NitrogenStyleConfig::tabStyle() const
{
  switch (_tabStyle->currentIndex())
  {
    case 0: return NitrogenStyleConfigData::TS_SINGLE;
    case 1:
    default:
    return NitrogenStyleConfigData::TS_PLAIN;
  }
}

//______________________________________________________
int NitrogenStyleConfig::menuBarAnimation() const
{
  switch (_menuBarAnimation->currentIndex())
  {
    case 0: return NitrogenStyleConfigData::MB_FADE;
    case 1: return NitrogenStyleConfigData::MB_FOLLOW_MOUSE;
    case 2: return NitrogenStyleConfigData::MB_NONE;
    default:
    return NitrogenStyleConfigData::MB_FADE;
  }
}

//______________________________________________________
int NitrogenStyleConfig::menuAnimation() const
{
  switch (_menuAnimation->currentIndex())
  {
    case 0: return NitrogenStyleConfigData::MA_FADE;
    case 1: return NitrogenStyleConfigData::MA_FOLLOW_MOUSE;
    case 2: return NitrogenStyleConfigData::MA_NONE;
    default:
    return NitrogenStyleConfigData::MA_FADE;
  }

}

//______________________________________________________
int NitrogenStyleConfig::toolBarAnimation() const
{
  switch (_toolBarAnimation->currentIndex())
  {
    case 0: return NitrogenStyleConfigData::MA_FADE;
    case 1: return NitrogenStyleConfigData::MA_FOLLOW_MOUSE;
    default:
    return NitrogenStyleConfigData::MA_FADE;
  }

}
