//////////////////////////////////////////////////////////////////////////////
// nitrogenanimations.cpp
// container for all animation engines
// -------------------
//
// Copyright (c) 2006, 2007 Riccardo Iaconelli <riccardo@kde.org>
// Copyright (c) 2006, 2007 Casper Boemann <cbr@boemann.dk>
// Copyright (c) 2009 Hugo Pereira Da Costa <hugo.pereira@free.fr>
//
// Permission is hereby granted, free of charge, to any person obtaining a copy
// of this software and associated documentation files (the "Software"), to
// deal in the Software without restriction, including without limitation the
// rights to use, copy, modify, merge, publish, distribute, sublicense, and/or
// sell copies of the Software, and to permit persons to whom the Software is
// furnished to do so, subject to the following conditions:
//
// The above copyright notice and this permission notice shall be included in
// all copies or substantial portions of the Software.
//
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
// AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
// FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS
// IN THE SOFTWARE.
//////////////////////////////////////////////////////////////////////////////

#include "nitrogenanimations.h"
#include "nitrogenanimations.moc"
#include "nitrogenstyleconfigdata.h"

namespace Nitrogen
{

  //____________________________________________________________
  Animations::Animations( QObject* parent ):
    QObject( parent ),
    abstractButtonEngine_( new GenericEngine( this ) ),
    lineEditEngine_( new GenericEngine( this ) ),
    menuBarEngine_( new MenuBarEngineV2( this ) ),
    menuEngine_( new MenuEngineV2( this ) ),
    scrollBarEngine_( new ScrollBarEngine( this ) ),
    sliderEngine_( new SliderEngine( this ) ),
    tabBarEngine_( new TabBarEngine( this ) ),
    toolBarEngine_( new ToolBarEngine( this ) )
  {
    //QTextStream( stdout ) << "Animations::Animations" << endl;
  }

  //____________________________________________________________
  void Animations::setupEngines( void )
  {

    //QTextStream( stdout ) << "Animations::setupEngines" << endl;

    // menu bar animation
    int menuBarAnimation( NitrogenStyleConfigData::menuBarAnimation() );
    if( menuBarAnimation == NitrogenStyleConfigData::MB_FADE && !qobject_cast<MenuBarEngineV1*>( menuBarEngine_ ) )
    {

      //QTextStream( stdout ) << "Animations::setupEngines - creating MenuBarEngineV1" << endl;
      menuBarEngine_->deleteLater();
      menuBarEngine_ = new MenuBarEngineV1( this );

    } else if( menuBarAnimation == NitrogenStyleConfigData::MB_FOLLOW_MOUSE && !qobject_cast<MenuBarEngineV2*>( menuBarEngine_ ) ) {

      //QTextStream( stdout ) << "Animations::setupEngines - creating MenuBarEngineV2" << endl;
      menuBarEngine_->deleteLater();
      menuBarEngine_ = new MenuBarEngineV2( this );

    }

    // menu bar animation
    int menuAnimation( NitrogenStyleConfigData::menuAnimation() );
    if( menuAnimation == NitrogenStyleConfigData::MA_FADE && !qobject_cast<MenuEngineV1*>( menuEngine_ ) )
    {

      //QTextStream( stdout ) << "Animations::setupEngines - creating MenuEngineV1" << endl;
      menuEngine_->deleteLater();
      menuEngine_ = new MenuEngineV1( this );

    } else if( menuAnimation == NitrogenStyleConfigData::MA_FOLLOW_MOUSE && !qobject_cast<MenuEngineV2*>( menuEngine_ ) ) {

      //QTextStream( stdout ) << "Animations::setupEngines - creating MenuEngineV2" << endl;
      menuEngine_->deleteLater();
      menuEngine_ = new MenuEngineV2( this );

    }

    // default enability, duration and maxFrame
    bool enabled( NitrogenStyleConfigData::animations() );
    int duration(  NitrogenStyleConfigData::animationsDuration() );

    QList<BaseEngine*> engines( findChildren<BaseEngine*>() );
    foreach( BaseEngine* engine, engines )
    {
      engine->setEnabled( enabled );
      engine->setDuration( duration );
      engine->setMaxFrame( 500 );
    }

    if( menuBarAnimation == NitrogenStyleConfigData::MB_NONE ) { menuBarEngine_->setEnabled( false ); }
    if( menuAnimation == NitrogenStyleConfigData::MA_NONE ) { menuEngine_->setEnabled( false ); }

    int toolBarAnimation( NitrogenStyleConfigData::toolBarAnimation() );
    if( toolBarAnimation == NitrogenStyleConfigData::TB_FADE ) { toolBarEngine_->setEnabled( false ); }

  }

}
