#ifndef nitrogenbasetransitionengine_h
#define nitrogenbasetransitionengine_h

//////////////////////////////////////////////////////////////////////////////
// nitrogenbasetransitionengine.h
// base widget for smooth transition handling
// -------------------
//
// Copyright (c) 2009 Hugo Pereira Da Costa <hugo.pereira@free.fr>
//
// Permission is hereby granted, free of charge, to any person obtaining a copy
// of this software and associated documentation files (the "Software"), to
// deal in the Software without restriction, including without limitation the
// rights to use, copy, modify, merge, publish, distribute, sublicense, and/or
// sell copies of the Software, and to permit persons to whom the Software is
// furnished to do so, subject to the following conditions:
//
// The above copyright notice and this permission notice shall be included in
// all copies or substantial portions of the Software.
//
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
// AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
// FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS
// IN THE SOFTWARE.
//////////////////////////////////////////////////////////////////////////////

#include <QtCore/QObject>
#include <QtCore/QPointer>

namespace Nitrogen
{

  class BaseTransitionEngine: public QObject
  {

    Q_OBJECT

    public:

    typedef QPointer<BaseTransitionEngine> Pointer;

    //! constructor
    BaseTransitionEngine( QObject* parent, int maxFrame, int duration ):
      QObject( parent ),
      enabled_( true ),
      maxFrame_( maxFrame ),
      duration_( duration )
    {}

    //! destructor
    virtual ~BaseTransitionEngine( void )
    {}

    //! enability
    virtual void setEnabled( bool value )
    { enabled_ = value; }

    //! enability
    virtual bool enabled( void ) const
    { return enabled_; }

    //! duration
    virtual void setDuration( int value )
    { duration_ = value; }

    //! duration
    virtual int duration( void ) const
    { return duration_; }

    //! max frame
    virtual void setMaxFrame( int value )
    { maxFrame_ = value; }

    //! max frame
    virtual int maxFrame( void ) const
    { return maxFrame_; }

    private:

    //! enability
    bool enabled_;

    //! max frame
    int maxFrame_;

    //! duration
    int duration_;

  };

}

#endif

