//////////////////////////////////////////////////////////////////////////////
// nitrogenscrollbardata.cpp
// stores event filters and maps widgets to timelines for animations
// -------------------
//
// Copyright (c) 2009 Hugo Pereira Da Costa <hugo.pereira@free.fr>
//
// Permission is hereby granted, free of charge, to any person obtaining a copy
// of this software and associated documentation files (the "Software"), to
// deal in the Software without restriction, including without limitation the
// rights to use, copy, modify, merge, publish, distribute, sublicense, and/or
// sell copies of the Software, and to permit persons to whom the Software is
// furnished to do so, subject to the following conditions:
//
// The above copyright notice and this permission notice shall be included in
// all copies or substantial portions of the Software.
//
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
// AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
// FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS
// IN THE SOFTWARE.
//////////////////////////////////////////////////////////////////////////////

#include "nitrogenscrollbardata.h"
#include "nitrogenscrollbardata.moc"

#include <QtGui/QHoverEvent>
#include <QtGui/QScrollBar>
#include <QtGui/QStyleOptionSlider>

Q_GUI_EXPORT QStyleOptionSlider qt_qscrollbarStyleOption(QScrollBar*);

namespace Nitrogen
{

  //______________________________________________
  bool ScrollBarData::eventFilter( QObject* object, QEvent* event )
  {

    // check event type
    switch( event->type() )
    {

      case QEvent::HoverEnter:
      case QEvent::HoverLeave:
      case QEvent::HoverMove:
      hoverMoveEvent( object, event );
      break;

      default: break;

    }

    return false;

  }

  //______________________________________________
  void ScrollBarData::hoverMoveEvent(  QObject* object, QEvent* event )
  {

    // try cast object to scrollbar
    QScrollBar* scrollBar( qobject_cast<QScrollBar*>( object ) );
    if( !scrollBar || scrollBar->isSliderDown() ) return;

    // retrieve scrollbar option
    QStyleOptionSlider opt( qt_qscrollbarStyleOption( qobject_cast<QScrollBar*>( object ) ) );

    // cast event
    QHoverEvent *he = static_cast<QHoverEvent*>(event);
    if( !he ) return;

    int hoverControl = scrollBar->style()->hitTestComplexControl(QStyle::CC_ScrollBar, &opt, he->pos(), scrollBar);
    if( hoverControl == QStyle::SC_ScrollBarSlider )
    {

      if( !hovered() ) {
        setHovered( true );
        timeLine()->setDirection( QTimeLine::Forward );
        if( !timeLine()->isRunning() ) timeLine()->start();
      }

    } else {

      if( hovered() )
      {
        setHovered( false );
        timeLine()->setDirection( QTimeLine::Backward );
        if( !timeLine()->isRunning() ) timeLine()->start();
      }

    }

    return;

  }

}
