#ifndef nitrogenstackedwidgettransitionengine_h
#define nitrogenstackedwidgettransitionengine_h

//////////////////////////////////////////////////////////////////////////////
// nitrogenstackedwidgettransitionengine.h
// handles smooth transitions for QStackedWidget
// -------------------
//
// Copyright (c) 2009 Hugo Pereira Da Costa <hugo.pereira@free.fr>
//
// Permission is hereby granted, free of charge, to any person obtaining a copy
// of this software and associated documentation files (the "Software"), to
// deal in the Software without restriction, including without limitation the
// rights to use, copy, modify, merge, publish, distribute, sublicense, and/or
// sell copies of the Software, and to permit persons to whom the Software is
// furnished to do so, subject to the following conditions:
//
// The above copyright notice and this permission notice shall be included in
// all copies or substantial portions of the Software.
//
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
// AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
// FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS
// IN THE SOFTWARE.
//////////////////////////////////////////////////////////////////////////////

#include "nitrogenbasetransitionengine.h"
#include "nitrogentransitionwidget.h"
#include "nitrogenwidgetgrab.h"

#include <QtCore/QPointer>
#include <QtGui/QStackedWidget>

namespace Nitrogen
{

  class StackedWidgetTransitionEngine: public BaseTransitionEngine
  {
    Q_OBJECT

    public:

    //! constructor
    StackedWidgetTransitionEngine( QStackedWidget* parent, int maxFrame, int duration );

    //! destructor
    virtual ~StackedWidgetTransitionEngine( void )
    {}

    //! enability
    virtual void setEnabled( bool enabled )
    {
      BaseTransitionEngine::setEnabled( enabled );
      transitionWidget_->setEnabled( enabled );
    }

    //! max frame
    virtual void setMaxFrame( int maxFrame )
    {
      BaseTransitionEngine::setMaxFrame( maxFrame );
      transitionWidget_->timeLine()->setFrameRange( 0, maxFrame );
    }

    //! duration
    virtual void setDuration( int duration )
    {
      BaseTransitionEngine::setDuration( duration );
      transitionWidget_->timeLine()->setDuration( duration );
    }

    //! event filter
    virtual bool eventFilter( QObject*, QEvent* );

    protected slots:

    //! create transition widget
    void createTransitionWidget( void );

    //! transition finished
    void transitionFinished( void );

    //! unregister widget
    void unregisterWidget( QObject* object )
    { if( object ) widgetGrabs_.remove( object ); }

    protected:

    //! toolbutton added
    virtual void childAddedEvent( QObject* );

    //! toolbutton added
    virtual void childShownEvent( QObject* );

    //! toolbutton added
    virtual void childHiddenEvent( QObject* );

    private:

    //! target
    QPointer<QStackedWidget> target_;

    //! transition widget
    TransitionWidget::Pointer transitionWidget_;

    //! widget grabber
    // WidgetGrab::Pointer widgetGrab_;

    //! map child to grabber
    typedef QMap< const QObject*, WidgetGrab::Pointer> WidgetMap;
    WidgetMap widgetGrabs_;

    //! last hidden object
    QPointer<QObject> lastHiddenObject_;

  };

}

#endif
