#ifndef nitrogentoolbardata_h
#define nitrogentoolbardata_h

//////////////////////////////////////////////////////////////////////////////
// nitrogentoolbardata.h
// stores event filters and maps widgets to timelines for animations
// -------------------
//
// Copyright (c) 2009 Hugo Pereira Da Costa <hugo.pereira@free.fr>
//
// Permission is hereby granted, free of charge, to any person obtaining a copy
// of this software and associated documentation files (the "Software"), to
// deal in the Software without restriction, including without limitation the
// rights to use, copy, modify, merge, publish, distribute, sublicense, and/or
// sell copies of the Software, and to permit persons to whom the Software is
// furnished to do so, subject to the following conditions:
//
// The above copyright notice and this permission notice shall be included in
// all copies or substantial portions of the Software.
//
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
// AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
// FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS
// IN THE SOFTWARE.
//////////////////////////////////////////////////////////////////////////////

#include "nitrogenwidgetdata.h"
#include <QtCore/QBasicTimer>

namespace Nitrogen
{

 //! toolbar data
  class ToolBarData: public WidgetData
  {

    Q_OBJECT

    public:

    //! constructor
    ToolBarData( QObject* parent, QWidget* target, int maxFrame, int duration );

    //! destructor
    virtual ~ToolBarData( void )
    {}

    //! event filter
    virtual bool eventFilter( QObject*, QEvent* );

    //! return timeLine associated to object at given position, if any
    virtual const TimeLine::Pointer& timeLine( void ) const
    { return timeLine_; }

    //! duration
    virtual void setDuration( int duration )
    { timeLine_->setDuration( duration ); }

    //! maxFrame
    virtual void setMaxFrame( int maxFrame )
    { timeLine_->setFrameRange( 0, maxFrame ); }

    //! return 'hover' rect position when widget is animated
    virtual const QRect& animatedRect( void ) const
    { return animatedRect_; }

    //! current rect
    virtual const QRect& currentRect( void ) const
    { return currentRect_; }

    //! timer
    const QBasicTimer& timer( void ) const
    { return timer_; }

    protected slots:

    //! updated animated rect
    virtual void updateAnimatedRect( int );

    protected:

    //! timer event
    virtual void timerEvent( QTimerEvent *);

    //!@name current object handling
    //@{

    //! guarded object pointer
    typedef const QObject* ObjectPointer;

    //! current object
    virtual const ObjectPointer& currentObject( void ) const
    { return currentObject_; }

    //! current object
    virtual void setCurrentObject( const QObject* object )
    { currentObject_ = ObjectPointer( object ); }

    //! current object
    virtual void clearCurrentObject( void )
    { currentObject_ = ObjectPointer(); }

    //@}

    //!@name rect handling
    //@{

    //! current rect
    virtual void setCurrentRect( const QRect& rect )
    { currentRect_ = rect; }

    //! current rect
    virtual void clearCurrentRect( void )
    { currentRect_ = QRect(); }

    //! previous rect
    virtual const QRect& previousRect( void ) const
    { return previousRect_; }

    //! previous rect
    virtual void setPreviousRect( const QRect& rect )
    { previousRect_ = rect; }

    //! previous rect
    virtual void clearPreviousRect( void )
    { previousRect_ = QRect(); }

    //! animated rect
    virtual void clearAnimatedRect( void )
    { animatedRect_ = QRect(); }

    //@}

    //! toolbar enterEvent
    virtual void enterEvent( const QObject* );

    //! toolbar enterEvent
    virtual void leaveEvent( const QObject* );

    //! toolbutton added
    virtual void childAddedEvent( QObject* );

    //! toolbutton enter event
    virtual void childEnterEvent( const QObject* );

    private:

    //! time line
    TimeLine::Pointer timeLine_;

    //! timer
    /*! this allows to add some delay before starting leaveEvent animation */
    QBasicTimer timer_;

    //! current object
    ObjectPointer currentObject_;

    // current rect
    QRect currentRect_;

    // previous rect
    QRect previousRect_;

    // animated rect
    QRect animatedRect_;

  };

}

#endif
