//////////////////////////////////////////////////////////////////////////////
// nitrogentoolbarengine.cpp
// stores event filters and maps widgets to timelines for animations
// -------------------
//
// Copyright (c) 2009 Hugo Pereira Da Costa <hugo.pereira@free.fr>
//
// Permission is hereby granted, free of charge, to any person obtaining a copy
// of this software and associated documentation files (the "Software"), to
// deal in the Software without restriction, including without limitation the
// rights to use, copy, modify, merge, publish, distribute, sublicense, and/or
// sell copies of the Software, and to permit persons to whom the Software is
// furnished to do so, subject to the following conditions:
//
// The above copyright notice and this permission notice shall be included in
// all copies or substantial portions of the Software.
//
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
// AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
// FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS
// IN THE SOFTWARE.
//////////////////////////////////////////////////////////////////////////////

#include "nitrogentoolbarengine.h"
#include "nitrogentoolbarengine.moc"

#include <QtCore/QEvent>

namespace Nitrogen
{

  //____________________________________________________________
  void ToolBarEngine::registerWidget( QWidget* widget )
  {

    if( !( enabled() && widget ) ) return;

    // create new data class
    if( !data_.contains( widget ) ) data_.insert( widget, QPointer<ToolBarData>( new ToolBarData( this, widget, maxFrame(), duration() ) ) );

    // connect destruction signal
    disconnect( widget, SIGNAL( destroyed( QObject* ) ), this, SLOT( unregisterWidget( QObject* ) ) );
    connect( widget, SIGNAL( destroyed( QObject* ) ), this, SLOT( unregisterWidget( QObject* ) ) );

  }

  //____________________________________________________________
  TimeLine::Pointer ToolBarEngine::timeLine( const QObject* object )
  {
    if( !enabled() ) return TimeLine::Pointer();
    QPointer<ToolBarData> out( data_.find( object ) );
    return out ? out->timeLine():TimeLine::Pointer();

  }

  //____________________________________________________________
  QRect ToolBarEngine::currentRect( const QObject* object )
  {
    if( !enabled() ) return QRect();
    QPointer<ToolBarData> out( data_.find( object ) );
    return out ? out->currentRect():QRect();

  }

  //____________________________________________________________
  QRect ToolBarEngine::animatedRect( const QObject* object )
  {
    if( !enabled() ) return QRect();
    QPointer<ToolBarData> out( data_.find( object ) );
    return out ? out->animatedRect():QRect();

  }

  //____________________________________________________________
  bool ToolBarEngine::isTimerActive( const QObject* object )
  {
    if( !enabled() ) return false;
    QPointer<ToolBarData> out( data_.find( object ) );
    return out ? out->timer().isActive():false;
  }

}
