//////////////////////////////////////////////////////////////////////////////
// nitrogenanimations.cpp
// container for all transition engines
// -------------------
//
// Copyright (c) 2009 Hugo Pereira Da Costa <hugo.pereira@free.fr>
//
// Permission is hereby granted, free of charge, to any person obtaining a copy
// of this software and associated documentation files (the "Software"), to
// deal in the Software without restriction, including without limitation the
// rights to use, copy, modify, merge, publish, distribute, sublicense, and/or
// sell copies of the Software, and to permit persons to whom the Software is
// furnished to do so, subject to the following conditions:
//
// The above copyright notice and this permission notice shall be included in
// all copies or substantial portions of the Software.
//
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
// AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
// FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS
// IN THE SOFTWARE.
//////////////////////////////////////////////////////////////////////////////

#include "nitrogentransitions.h"
#include "nitrogentransitions.moc"
#include "nitrogenstackedwidgettransitionengine.h"
#include "nitrogenstyleconfigdata.h"
#include "nitrogenwidgetgrab.h"

namespace Nitrogen
{

  //___________________________________________________________
  Transitions::Transitions( QObject* parent ):
    QObject( parent ),
    enabled_( true ),
    maxFrame_( 500 ),
    duration_( 150 )
  {}

  //___________________________________________________________
  void Transitions::registerStackedWidget( QStackedWidget* widget )
  {

    // do nothing if disabled
    if( !( enabled() && widget ) ) return;

    // don't register if widget already in map
    if( engines_.contains( widget ) ) return;

    BaseTransitionEngine* engine( new StackedWidgetTransitionEngine( widget, maxFrame(), duration() ) );
    engine->setEnabled( enabled() );
    engines_.insert( widget, engine );

    connect( widget, SIGNAL( destroyed( QObject* ) ), SLOT( unregisterObject( QObject* ) ) );

  }

  //___________________________________________________________
  void Transitions::setupEngines( void )
  {

    // default enability, duration and maxFrame
    // enabled_ = NitrogenStyleConfigData::animations();
    setEnabled( NitrogenStyleConfigData::animations() && NitrogenStyleConfigData::stackedWidgetAnimations() );
    setDuration( NitrogenStyleConfigData::animationsDuration() );
    setMaxFrame( 500 );

    foreach( const BaseTransitionEngine::Pointer& engine, engines_ )
    {
      if( engine )
      {
        engine->setEnabled( enabled() );
        engine->setDuration( duration() );
        engine->setMaxFrame( maxFrame() );
      }
    }

  }

  //___________________________________________________________
  void Transitions::unregisterObject( QObject* object )
  {
    if( !object ) return;
    engines_.remove( object );
  }

}
