#ifndef nitrogentransitions_h
#define nitrogentransitions_h

//////////////////////////////////////////////////////////////////////////////
// nitrogenanimations.h
// container for all transition engines
// -------------------
//
// Copyright (c) 2009 Hugo Pereira Da Costa <hugo.pereira@free.fr>
//
// Permission is hereby granted, free of charge, to any person obtaining a copy
// of this software and associated documentation files (the "Software"), to
// deal in the Software without restriction, including without limitation the
// rights to use, copy, modify, merge, publish, distribute, sublicense, and/or
// sell copies of the Software, and to permit persons to whom the Software is
// furnished to do so, subject to the following conditions:
//
// The above copyright notice and this permission notice shall be included in
// all copies or substantial portions of the Software.
//
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
// AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
// FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS
// IN THE SOFTWARE.
//////////////////////////////////////////////////////////////////////////////

#include "nitrogenbasetransitionengine.h"

#include <QtCore/QObject>
#include <QtCore/QMap>
#include <QtGui/QStackedWidget>

namespace Nitrogen
{

  //! stores engines
  class Transitions: public QObject
  {

    Q_OBJECT

    public:

    //! constructor
    explicit Transitions( QObject* );

    //! destructor
    virtual ~Transitions( void )
    {}

    //! register stacked widget
    virtual void registerStackedWidget( QStackedWidget* );

    //! enability
    virtual void setEnabled( bool value )
    { enabled_ = value; }

    //! enability
    virtual bool enabled( void ) const
    { return enabled_; }

    //! max frame
    virtual void setMaxFrame( int value )
    { maxFrame_ = value; }

    //! max frame
    virtual int maxFrame( void ) const
    { return maxFrame_; }

    //! duration
    virtual void setDuration( int value )
    { duration_ = value; }

    //! duration
    virtual int duration( void ) const
    { return duration_; }

    public slots:

    //! setup engines
    virtual void setupEngines( void );

    protected slots:

    //! unregister engine
    virtual void unregisterObject( QObject* );

    private:

    //! enability
    bool enabled_;

    //! max frame
    int maxFrame_;

    //! duration
    int duration_;

    //! register engines
    typedef QMap< const QObject*, BaseTransitionEngine::Pointer > EngineMap;

    //! register engines
    EngineMap engines_;

  };

}

#endif
