//////////////////////////////////////////////////////////////////////////////
// nitrogentransitionwidget.cpp
// widget used to clone widgets temporarily in order to simulate smooth transitions
// between two widget states
// -------------------
//
// Copyright (c) 2009 Hugo Pereira Da Costa <hugo.pereira@free.fr>
//
// Permission is hereby granted, free of charge, to any person obtaining a copy
// of this software and associated documentation files (the "Software"), to
// deal in the Software without restriction, including without limitation the
// rights to use, copy, modify, merge, publish, distribute, sublicense, and/or
// sell copies of the Software, and to permit persons to whom the Software is
// furnished to do so, subject to the following conditions:
//
// The above copyright notice and this permission notice shall be included in
// all copies or substantial portions of the Software.
//
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
// AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
// FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS
// IN THE SOFTWARE.
//////////////////////////////////////////////////////////////////////////////

#include "nitrogentransitionwidget.h"
#include "nitrogentransitionwidget.moc"

#include <QtGui/QPainter>

namespace Nitrogen
{

  //___________________________________________________________________
  TransitionWidget::TransitionWidget( QWidget *parent, int maxFrame, int duration ):
    QWidget( parent ),
    timeLine_( new TimeLine( duration, this ) )
  {

    timeLine()->setFrameRange( 0, maxFrame );
    timeLine()->setCurveShape( QTimeLine::EaseInOutCurve );
    timeLine()->setDirection( QTimeLine::Forward );
    connect( timeLine().data(), SIGNAL( frameChanged( int ) ), this, SLOT( update( void )) );

  }

  //___________________________________________________________________
  TransitionWidget::~TransitionWidget()
  {}

  //___________________________________________________________________
  bool TransitionWidget::initialize( QPixmap pixmap, QRect rect )
  {

    // resize
    setGeometry( rect.translated( -rect.topLeft() ) );

    // grab pixmap
    pixmap_ = QPixmap( rect.size() );
    pixmap_.fill( Qt::transparent );
    QPainter painter( &pixmap_ );
    painter.drawPixmap( QPoint(0,0), pixmap, rect );
    painter.end();

    return true;

  }

  //___________________________________________________________________
  void TransitionWidget::paintEvent( QPaintEvent* event )
  {

    // create painter
    QPainter painter( this );
    painter.setClipRect( event->rect() );
    painter.fillRect( rect(), Qt::transparent );
    painter.setRenderHints(QPainter::SmoothPixmapTransform);

    // draw pixmap
    if( timeLine()->isRunning() ) painter.setOpacity( 1.0-timeLine()->ratio() );
    painter.drawPixmap( QPoint(0,0), pixmap_ );
    painter.end();

  }

}
