#ifndef nitrogentransitionwidget_h
#define nitrogentransitionwidget_h

//////////////////////////////////////////////////////////////////////////////
// nitrogentransitionwidget.h
// widget used to clone widgets temporarily in order to simulate smooth transitions
// between two widget states
// -------------------
//
// Copyright (c) 2009 Hugo Pereira Da Costa <hugo.pereira@free.fr>
//
// Permission is hereby granted, free of charge, to any person obtaining a copy
// of this software and associated documentation files (the "Software"), to
// deal in the Software without restriction, including without limitation the
// rights to use, copy, modify, merge, publish, distribute, sublicense, and/or
// sell copies of the Software, and to permit persons to whom the Software is
// furnished to do so, subject to the following conditions:
//
// The above copyright notice and this permission notice shall be included in
// all copies or substantial portions of the Software.
//
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
// AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
// FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS
// IN THE SOFTWARE.
//////////////////////////////////////////////////////////////////////////////

#include "nitrogentimeline.h"

#include <QtCore/QPointer>
#include <QtGui/QPaintEvent>
#include <QtGui/QPixmap>
#include <QtGui/QWidget>

namespace Nitrogen
{
  class TransitionWidget: public QWidget
  {

    Q_OBJECT

    public:

    typedef QPointer<TransitionWidget> Pointer;

    //! constructor
    TransitionWidget( QWidget*, int maxFrame, int duration );

    //! destructor
    virtual ~TransitionWidget();

    //! set starting widget
    bool initialize( QPixmap, QRect rect );

    //! start
    void start( void ) const
    {
      if( !timeLine()->isRunning() ) timeLine()->start();
    }

    //! timeline
    const TimeLine::Pointer& timeLine( void ) const
    { return timeLine_; }

    //! pixmap
    const QPixmap& pixmap( void ) const
    { return pixmap_; }

    protected:

    //! mouse press event
    /*!
    on mouse press, widget is automatically hidden. This prevents cases where "normal"
    hiding (at end of animation) fails for whatever reason
    */
    virtual void mousePressEvent( QMouseEvent* )
    { if( isVisible() ) hide(); }

    //! paint event
    virtual void paintEvent( QPaintEvent* );

    private:

    //! timeline
    TimeLine::Pointer timeLine_;

    //! current widget pixmap
    QPixmap pixmap_;

  };

}
#endif
