//////////////////////////////////////////////////////////////////////////////
// WidgetGrab.cpp
// redirect widget painting to a pixmap
// -------------------
//
// Copyright (c) 2009 Hugo Pereira Da Costa <hugo.pereira@free.fr>
//
// Permission is hereby granted, free of charge, to any person obtaining a copy
// of this software and associated documentation files (the "Software"), to
// deal in the Software without restriction, including without limitation the
// rights to use, copy, modify, merge, publish, distribute, sublicense, and/or
// sell copies of the Software, and to permit persons to whom the Software is
// furnished to do so, subject to the following conditions:
//
// The above copyright notice and this permission notice shall be included in
// all copies or substantial portions of the Software.
//
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
// AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
// FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS
// IN THE SOFTWARE.
//////////////////////////////////////////////////////////////////////////////

#include "nitrogenwidgetgrab.h"
#include "nitrogenwidgetgrab.moc"

#include <QtCore/QEvent>
#include <QtCore/QTextStream>
#include <QtGui/QResizeEvent>
#include <QtGui/QPaintEvent>
#include <QtGui/QPainter>

namespace Nitrogen
{

  //_________________________________________
  WidgetGrab::WidgetGrab( QWidget* parent ):
    QObject( parent ),
    recursiveCheck_( false )
  {
    if( parent )
    {
      // initialize pixmap
      pixmap_ = QPixmap( parent->size() );
      pixmap_.fill(Qt::transparent);

      // install event filter
      parent->installEventFilter( this );
    }
  }

  //_________________________________________
  bool WidgetGrab::eventFilter( QObject* object, QEvent* event)
  {

    if( object == parent() )
    {

      switch( event->type() )
      {
        case QEvent::Resize:
        {
          QResizeEvent* re( static_cast<QResizeEvent*>(event) );
          pixmap_ = QPixmap( re->size() );
          pixmap_.fill( Qt::transparent );
          break;
        }

        case QEvent::Paint:
        {
          QPaintEvent* pe( static_cast<QPaintEvent*>(event) );
          clipRect_ = pe->rect();
          if( enabled() && !recursiveCheck_ ) timer_.start( 0, this );
          break;

        }

        default: break;

      }

    }

    return QObject::eventFilter( object, event );

  }

  //_________________________________________
  void WidgetGrab::timerEvent( QTimerEvent* event )
  {
    if( event->timerId() != timer_.timerId() ) return QObject::timerEvent( event );
    timer_.stop();

    if( !enabled() ) return;
    QWidget* widget( qobject_cast<QWidget*>(parent() ) );
    if( pixmap_.isNull() || !widget ) return;

    recursiveCheck_ = true;
    widget->render( &pixmap_ );
    recursiveCheck_ = false;

  }

}
