#ifndef nitrogenwidgetgrab_h
#define nitrogenwidgetgrab_h

//////////////////////////////////////////////////////////////////////////////
// nitrogenwidgetgrab.h
// redirect widget painting to a pixmap
// -------------------
//
// Copyright (c) 2009 Hugo Pereira Da Costa <hugo.pereira@free.fr>
//
// Permission is hereby granted, free of charge, to any person obtaining a copy
// of this software and associated documentation files (the "Software"), to
// deal in the Software without restriction, including without limitation the
// rights to use, copy, modify, merge, publish, distribute, sublicense, and/or
// sell copies of the Software, and to permit persons to whom the Software is
// furnished to do so, subject to the following conditions:
//
// The above copyright notice and this permission notice shall be included in
// all copies or substantial portions of the Software.
//
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
// AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
// FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS
// IN THE SOFTWARE.
//////////////////////////////////////////////////////////////////////////////

#include <QtCore/QBasicTimer>
#include <QtCore/QObject>
#include <QtCore/QPointer>
#include <QtGui/QPixmap>
#include <QtGui/QWidget>

namespace Nitrogen
{

  class WidgetGrab: public QObject
  {

    Q_OBJECT

    public:

    typedef QPointer<WidgetGrab> Pointer;

    //! constructor
    WidgetGrab( QWidget* parent );

    //! destructor
    virtual ~WidgetGrab( void )
    {}

    //! event filter
    virtual bool eventFilter( QObject*, QEvent* );

    //! pixmap
    QPixmap pixmap( void ) const
    { return pixmap_; }

    //! enability
    bool enabled( void ) const
    { return enabled_; }

    //! enability
    void setEnabled( bool value )
    { enabled_ = value; }

    protected:

    //! timer event
    virtual void timerEvent( QTimerEvent* );

    private:

    //! enability
    bool enabled_;

    //! true when recording is enabled
    bool recursiveCheck_;

    //! pixmap
    QPixmap pixmap_;

    //! timer
    /*! it is needed so that the grab happens outside of the paintEvent */
    QBasicTimer timer_;

    //! paintevent clipRect
    QRect clipRect_;

  };

}

#endif
